/*
    Copyright (c) 2013 aizo ag, Zurich, Switzerland

    This file is part of libdsuid.

    libdsuid is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libdsuid is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include <check.h>

#include "dsuid.h"

START_TEST(test_dsuid_from_dsid)
{
    /* test dsuid_from_dsid */
    dsid_t dsid = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07 } };
    dsuid_t dsuid;
    dsuid_t dsuid_expected = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x00 } };

    dsuid = dsuid_from_dsid(&dsid);
    if (IsEqualDsuid(dsuid, dsuid_expected)) {
        /* test passed */
    } else {
        uint8_t i;
        char *p;
        char dsuid_str[DSUID_STR_LEN], dsuid_exp_str[DSUID_STR_LEN];

        p = dsuid_str;
        for (i = 0; i < DSUID_SIZE; i++) {
            p += sprintf(p, "%02x", dsuid.id[i]);
        }
        p = '\0';
        p = dsuid_exp_str;
        for (i = 0; i < DSUID_SIZE; i++) {
            p += sprintf(p, "%02x", dsuid_expected.id[i]);
        }
        p = '\0';
        ck_assert_msg(IsEqualDsuid(dsuid, dsuid_expected) == 0,
            "dsuid convertion failed.\n"
            "expected dsuid:  %s\n"
            "converted dsuid: %s\n", dsuid_exp_str, dsuid_str);
    }
}
END_TEST

START_TEST(test_dsuid_to_dsid)
{
    int rval;
    size_t i;
    char *p;
    char dsid_str[sizeof(dsid_t) * 2 + 1], dsid_exp_str[sizeof(dsid_t) * 2 + 1];
    dsid_t dsid;
    dsuid_t dsuid1   = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };
    dsid_t dsid1_exp = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d } };
    dsuid_t dsuid2   = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };

    rval = dsuid_to_dsid(&dsuid1, &dsid);

    p = dsid_str;
    for (i = 0; i < sizeof(dsid_t); i++) {
        p += sprintf(p, "%02x", dsid.id[i]);
    }
    p = '\0';
    p = dsid_exp_str;
    for (i = 0; i < sizeof(dsid_t); i++) {
        p += sprintf(p, "%02x", dsid1_exp.id[i]);
    }
    p = '\0';
    ck_assert_msg((rval == DSUID_RC_OK) && (memcmp(&dsid1_exp, &dsid, sizeof(dsid_t)) == 0),
        "dsuid to dsid 1 failed.\n"
        "return value  : %s\n"
        "dsid expected : %s\n"
        "dsid          : %s\n", dsuid_rc_to_text(rval), dsid_exp_str, dsid_str);

    rval = dsuid_to_dsid(&dsuid2, &dsid);
    ck_assert_msg(rval == DSUID_RC_INV_DSUID,
        "dsuid to dsid 2 failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));
}
END_TEST

START_TEST(test_get_serial_number)
{
    /* test dsuid_get_serial_number */
    int rval;
    uint32_t serialNumber;
    dsuid_t dsuid1 = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };
    dsuid_t dsuid2 = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x01, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };

    rval = dsuid_get_serial_number(&dsuid1, &serialNumber);
    ck_assert_msg((rval == DSUID_RC_OK) && (serialNumber == 0x0a0b0c0d),
        "get serial number 1 failed.\n"
        "return value:  %s\n"
        "serial number: 0x%08x\n", dsuid_rc_to_text(rval), serialNumber);

    rval = dsuid_get_serial_number(&dsuid2, &serialNumber);
    ck_assert_msg(rval == DSUID_RC_INV_DSUID,
        "get serial number 2 failed.\n"
        "return value:  %s\n"
        "serial number: %d\n", dsuid_rc_to_text(rval), serialNumber);
}
END_TEST

START_TEST(test_dsuid_to_string)
{
    int rval;
    char dsuid_str[DSUID_STR_LEN];
    char dsuid_expected_str[DSUID_STR_LEN] = "0102030405060708090a0b0c0d0e0f1011";
    dsuid_t dsuid = { { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11 } };

    rval = dsuid_to_string(&dsuid, dsuid_str);
    ck_assert_msg((rval == DSUID_RC_OK) && (strcmp(dsuid_str, dsuid_expected_str) == 0),
        "string to dsuid failed.\n"
        "return value  : %d\n"
        "dsuid         : %s\n"
        "dsuid expected: %s\n", rval, dsuid_str, dsuid_expected_str);
}
END_TEST

START_TEST(test_dsuid_from_string)
{
    int rval;
    char dsuid_str[DSUID_STR_LEN] = "0102030405060708090a0b0c0d0e0f1011";
    dsuid_t dsuid;
    dsuid_t dsuid_expected = { { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11 } };


    rval = dsuid_from_string(dsuid_str, &dsuid);
    ck_assert_msg((rval == DSUID_RC_OK) && IsEqualDsuid(dsuid, dsuid_expected),
        "dsuid from string failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));
}
END_TEST

START_TEST(test_get_next_dsuid)
{
    int rval;
    dsuid_t dsuid;
    dsuid_t dsuid1      = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };
    dsuid_t dsuid1_next = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0e, 0x00 } };
    dsuid_t dsuid2      = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0xff, 0x00 } };
    dsuid_t dsuid2_next = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0d, 0x00, 0x00 } };
    dsuid_t dsuid3      = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0xff, 0xff, 0x00 } };
    dsuid_t dsuid3_next = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0c, 0x00, 0x00, 0x00 } };
    dsuid_t dsuid4      = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0xff, 0xff, 0xff, 0x00 } };
    dsuid_t dsuid4_next = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0b, 0x00, 0x00, 0x00, 0x00 } };
    dsuid_t dsuid5      = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0xff, 0xff, 0xff, 0xff, 0x00 } };
    dsuid_t dsuid6      = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };

    rval = dsuid_get_next_dsuid(&dsuid1, &dsuid);
    ck_assert_msg((rval == DSUID_RC_OK) && IsEqualDsuid(dsuid, dsuid1_next),
        "get next dsuid 1 failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));

    rval = dsuid_get_next_dsuid(&dsuid2, &dsuid);
    ck_assert_msg((rval == DSUID_RC_OK) && IsEqualDsuid(dsuid, dsuid2_next),
        "get next dsuid 2 failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));

    rval = dsuid_get_next_dsuid(&dsuid3, &dsuid);
    ck_assert_msg((rval == DSUID_RC_OK) && IsEqualDsuid(dsuid, dsuid3_next),
        "get next dsuid 3 failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));

    rval = dsuid_get_next_dsuid(&dsuid4, &dsuid);
    ck_assert_msg((rval == DSUID_RC_OK) && IsEqualDsuid(dsuid, dsuid4_next),
        "get next dsuid 4 failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));

    rval = dsuid_get_next_dsuid(&dsuid5, &dsuid);
    ck_assert_msg(rval == DSUID_RC_INV_SERIAL,
        "get next dsuid 5 failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));

    rval = dsuid_get_next_dsuid(&dsuid6, &dsuid);
    ck_assert_msg(rval == DSUID_RC_INV_DSUID,
        "get next dsuid 6 failed.\n"
        "return value: %s\n", dsuid_rc_to_text(rval));
}
END_TEST

START_TEST(test_dsuid_generate_from_namespace)
{
    int rval;
    char dsuid1_str[DSUID_STR_LEN], dsuid2_str[DSUID_STR_LEN];
    dsuid_t dsuid1, dsuid2;

    /* test consistent generation */
    rval = dsuid_generate_from_namespace(DSUID_NS_ENOCEAN, "test device", &dsuid1);
    dsuid_to_string(&dsuid1, dsuid1_str);
    ck_assert_msg(rval == DSUID_RC_OK,
        "dsuid generate 1 failed.\n"
        "return value : %s\n"
        "dsuid        : %s\n", dsuid_rc_to_text(rval), dsuid1_str);

    rval = dsuid_generate_from_namespace(DSUID_NS_ENOCEAN, "test device", &dsuid2);
    dsuid_to_string(&dsuid2, dsuid2_str);
    ck_assert_msg(rval == DSUID_RC_OK,
        "dsuid generate 2 failed.\n"
        "return value : %s\n"
        "dsuid        : %s\n", dsuid_rc_to_text(rval), dsuid2_str);

    ck_assert_msg(IsEqualDsuid(dsuid1, dsuid2),
        "dsuid generate comparison failed.\n"
        "dsuid1: %s\n"
        "dsuid2: %s\n", dsuid1_str, dsuid2_str);

    /* test wrong parameter */
    rval = dsuid_generate_from_namespace(-1, "test device", &dsuid1);
    ck_assert_msg(rval == DSUID_RC_WRONG_PAR,
        "dsuid generate wrong parameter 1 failed.\n"
        "return value : %s\n", dsuid_rc_to_text(rval));

    rval = dsuid_generate_from_namespace(DSUID_NS_VDSM, NULL, &dsuid1);
    ck_assert_msg(rval == DSUID_RC_WRONG_PAR,
        "dsuid generate wrong parameter 2 failed.\n"
        "return value : %s\n", dsuid_rc_to_text(rval));

    /* test different name */
    rval = dsuid_generate_from_namespace(DSUID_NS_ENOCEAN, "test device2", &dsuid1);
    dsuid_to_string(&dsuid1, dsuid1_str);
    ck_assert_msg((rval == DSUID_RC_OK) && !IsEqualDsuid(dsuid1, dsuid2),
        "dsuid generate different name failed.\n"
        "return value : %s\n"
        "dsuid1: %s\n"
        "dsuid2: %s\n", dsuid_rc_to_text(rval), dsuid1_str, dsuid2_str);

    /* test different namespace */
    rval = dsuid_generate_from_namespace(DSUID_NS_VDC, "test device", &dsuid1);
    dsuid_to_string(&dsuid1, dsuid1_str);
    ck_assert_msg((rval == DSUID_RC_OK) && !IsEqualDsuid(dsuid1, dsuid2),
        "dsuid generate different namespace failed.\n"
        "return value : %s\n"
        "dsuid1: %s\n"
        "dsuid2: %s\n", dsuid_rc_to_text(rval), dsuid1_str, dsuid2_str);
}
END_TEST

Suite *dsuid_suite()
{
    Suite *s = suite_create("dsuid");
    TCase *tc_dsuid = tcase_create("dsuid");

    tcase_set_timeout(tc_dsuid, 4);

    tcase_add_test(tc_dsuid, test_dsuid_from_dsid);
    tcase_add_test(tc_dsuid, test_dsuid_to_dsid);
    tcase_add_test(tc_dsuid, test_get_serial_number);
    tcase_add_test(tc_dsuid, test_dsuid_to_string);
    tcase_add_test(tc_dsuid, test_dsuid_from_string);
    tcase_add_test(tc_dsuid, test_get_next_dsuid);
    tcase_add_test(tc_dsuid, test_dsuid_generate_from_namespace);
    suite_add_tcase(s, tc_dsuid);
    return s;
}

int main(int argc, char **argv)
{
    (void)argc;
    (void)argv;

    Suite *dsuid = dsuid_suite();
    SRunner *test_runner = srunner_create(dsuid);
    srunner_run_all(test_runner, CK_NORMAL);
    int failed = 0;
    failed = srunner_ntests_failed(test_runner);
    srunner_free(test_runner);
    return failed;
}
